#!/usr/bin/perl 
#
# DomainAdmin:
#
# Copyright (c) 2000 Paul Warren. All rights reserved.
#
# $Log: DomainAdmin.pm,v $
# Revision 1.5  2002/01/09 21:29:39  pdw
# Removed obsolete reference to MD5 module.
# Fixed script to set maximum number of POP3 boxes.
#
# Revision 1.4  2001/07/23 22:08:15  pdw
# Lots of changes including:
# scripts/* now use PasswordCrypt
# Added default_hash_method and allow_plaintext options.
# Makefile should now make dist properly.
#
# Revision 1.3  2001/07/17 22:12:57  pdw
# Refactoring DB code.  Non-working state.
#
# Revision 1.2  2001/07/17 20:54:21  pdw
# Edging towards 0.4...
#
# Revision 1.1  2001/06/24 21:12:02  pdw
# Moved lib directory.
#
# Revision 1.4  2001/04/07 22:47:12  pdw
# Removed debug code.
#
# Revision 1.3  2001/04/07 22:46:34  pdw
# Added escaping of URLs, as per rfc1738
#
# Revision 1.2  2001/04/07 18:30:11  pdw
# Added config variable sqldb to specify database name.
#
# Revision 1.1.1.1  2000/11/18 21:35:09  pdw
# Initial commit
#
#

package DomainAdmin;

require DomainAdminConfig;
require DomainAdminDB;

use DBI;
use DBD::mysql;

$login_latency = "900"; # expire cookies 10 minutes after last action.

sub http_head {
    my @cookies = @_;
    
    print "Content-type: text/html\n";

    foreach $cookie (@cookies) {
        print <<EOF;
Set-cookie: $cookie 
EOF
    }
    print "\n";
}

sub html_head {
    my ($title, $headline) = @_;

    print <<EOF;
<html>

<head>
<title>
$title
</title>
<link rel="stylesheet" type="text/css" href=$DomainAdminConfig::css>
</head>
<body>
<h1 class="ptitle">
$headline
</h1>

EOF
}

sub html_navbar_start {
    print <<EOF;
<HR>
<P>
<SPAN class="navbar">
EOF
}

sub html_navbar_end {
    print <<EOF;
</SPAN>
</P>
<P class="smallprint">
$DomainAdminConfig::author
</P>
EOF
}

sub html_navbar_logout {
    print <<EOF;
<A HREF="logout">Logout</A>
EOF
}

sub html_navbar_domain {
    print <<EOF;
<A HREF="login">Domain Control Panel</A> |
EOF
    html_navbar_logout();
}

sub html_navbar_email {
    print <<EOF;
<A HREF="email_setup">Email Control Panel</A> |
EOF
    html_navbar_domain();
}

sub html_domain_footer {
    print <<EOF;
<HR>
<SPAN class="navbar">
<A HREF="login">Domain Control Panel</A> |
<A HREF="logout">Logout</A>
</SPAN>
EOF
}

sub html_tail { 
    print <<EOF;

</body>
</html>
EOF
}

sub error_internal_error {
    my $err = shift;
    http_head();
    html_head($err,"Internal Error");
    print <<EOF;
<p class="info">
It would appear that something has gone wrong, probably at our end. You could try emailing us and quoting the error:
</p>
<p class="errormsg">
$err
</p>
EOF
    html_tail();
    exit;
}

#
# escapes characters, as per rfc1738 section 2.2
#
sub url_escape {
    my $text = shift;
    $text =~ s/([^a-zA-Z0-9\$_.+!*'()-])/sprintf('%%%2X',ord($1))/ge;
    return $text;
}


sub http_unescape {
    my $text = shift;
    
    $text =~ s/&lt;/</g;
    $text =~ s/&gt;/>/g;

    $text =~ s/&amp;/&/g;
    $text =~ s/\%(\d+)/chr(hex($1))/ge;

    return $text;
}

sub cgi_getparams {
    undef %param;
    # process POST arguments
    read(STDIN, $a, $ENV{CONTENT_LENGTH});

    # process GET arguments
    $a .= "&".$ENV{QUERY_STRING};

    @p = split('&', $a);

    foreach $q (@p) {
        ($n, $v) = $q =~ /^([^=]+)=(.+)$/;
        $param{http_unescape($n)} = http_unescape($v);
    }
}

sub db_open() {
    if ($db eq undef) {
        $db = new DomainAdminDB();
        if(!$db->connect()) {
            error_internal_error("Couldn't open database");
        }
    }
    # TODO: refactor properly and remove this hack:
    $dbh = $db->dbh();

}

sub db_retrieve_domain {
    my $domain = shift;
    my $dbh = $db->{dbh};
    my $dd = $dbh->quote($domain); 
    my $stmt = <<EOF;
SELECT
    *
FROM
    domain
WHERE
    domain_name = $dd
EOF
    my $sth = $dbh->prepare($stmt);
    if ($sth->execute() != 1) {
        return undef;
    } else {
        return $sth->fetchrow_hashref();
    }
}

sub session_set_id {
    my ($domain, $id) = @_;
    my $dbh = $db->dbh();
    my $ii = $dbh->quote($id);
    my $dd = $dbh->quote($domain);
    my $expires = time() + $login_latency;
    my $ee = $dbh->quote($expires);
    my $stmt = <<EOF;
REPLACE INTO
    web_session (domain_name, session_id, expires)
VALUES
    ($dd, $ii, $ee)
EOF
    if (!$dbh->do($stmt)) {
        DomainAdmin::error_internal_error("Could not save session id");
    }
}

sub cookies_get_cookies {
    undef %cookies;
    @p = split(/[; ]/, $ENV{HTTP_COOKIE});
    foreach $q (@p) {
        ($n, $v) = $q =~ /^([^=]+)=(.+)$/;
        $cookies{http_unescape($n)} = http_unescape($v);
    }
}

sub authenticate_cookie {
    undef $failed_login;
    if (defined $cookies{sessionid}) {
        my $ar = $db->retrieve_session($cookies{sessionid});
        if (!defined $ar) {
            $failed_login = $db->error();
            return 0;
        }
        my ($domain, $id, $expires) = @$ar;
        if (!$domain) {
            $failed_login = "invalid session cookie";
            return 0;
        }
        if (time() > $expires) {
            $failed_login = "your session cookie has expired";
            return 0;
        }
        session_set_id($domain,$id);
	$DomainAdmin::domain = $domain;
        return 1;    
    }
    $failed_login = "Could not find session cookie";
}

cgi_getparams();
db_open();

unless ($noprocesscookies) { 
    cookies_get_cookies;    
    authenticate_cookie();
    unless ($nocheckcookie) {
        if (defined $failed_login) {
            http_head();
            html_head("Domain Control Panel","Authentication Error");
            print <<EOF;
<p class="error">
$failed_login
</p>
<p class="info">
Please <A HREF="login">login</A> again.
</p>
EOF
            html_tail();
            exit;
        }
    }
}

1;
